/*
 * Copyright (c) 2013 Peter S. May
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
 * associated documentation files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
 * NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package us.wxy.iomux;

import java.io.IOException;
import java.io.InputStream;

/**
 * Wrapper for using {@link ByteIOInput} as {@link InputStream}.
 * 
 * Only reading is implemented; optional features are left unimplemented and {@link #close} is a
 * no-op.
 */
public class ByteIOInputStream extends InputStream {
  protected ByteIOInput in;

  public ByteIOInputStream(ByteIOInput in) {
    this.in = in;
  }

  @Override
  public int read() throws IOException {
    byte[] b = new byte[1];
    int n = in.readBytes(b, 0, 1);
    return (n <= 0) ? -1 : (0xFF & b[0]);
  }

  @Override
  public int read(byte[] b, int off, int len) throws IOException {
    int n = in.readBytes(b, off, len);
    return (n <= 0) ? -1 : n;
  }
}
