/*
 * Copyright (c) 2013 Peter S. May
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
 * associated documentation files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
 * NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package us.wxy.iomux;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;


/**
 * Extension of {@link ByteArrayInput} to include transfers to output streams and byte outputs.
 */
public interface ByteInput extends ByteArrayInput {

  /**
   * Reads data out of this object into a {@link ByteArrayOutput}. This is equivalent to
   * {@code getBytes(dest, Integer.MAX_VALUE)}.
   * 
   * @param dest An {@link ByteArrayOutput} to copy data into
   * @return The number of bytes actually copied
   */
  public abstract int getBytes(ByteArrayOutput dest);

  /**
   * Reads data out of this object into a {@link ByteArrayOutput}.
   * 
   * @param dest An {@link ByteArrayOutput} to copy data into
   * @param length The maximum number of bytes to attempt to copy
   * @return The number of bytes actually copied, which may be {@code length} or less, including 0,
   *         if less data is available
   */
  public abstract int getBytes(ByteArrayOutput dest, int length);

  /**
   * Reads data out of this object into an output stream. This is equivalent to
   * {@code getBytes(dest, Integer.MAX_VALUE)}.
   * 
   * @param dest An {@link OutputStream} to copy data into
   * @return The number of bytes actually copied
   * @throws IOException If an I/O error occurs while writing to {@code dest}
   */
  public abstract int getBytes(OutputStream dest) throws IOException;

  /**
   * Reads data out of this object into an output stream.
   * 
   * @param dest An {@link OutputStream} to copy data into
   * @param length The maximum number of bytes to attempt to copy
   * @return The number of bytes actually copied, which may be {@code length} or less, including 0,
   *         if less data is available
   * @throws IOException If an I/O error occurs while writing to {@code dest}
   */
  public abstract int getBytes(OutputStream dest, int length) throws IOException;

  /**
   * Reads data out of this object into an print stream. This is equivalent to
   * {@code getBytes(dest, Integer.MAX_VALUE)}.
   * 
   * @param dest An {@link PrintStream} to copy data into
   * @return The number of bytes actually copied
   */
  public abstract int getBytes(PrintStream dest);

  /**
   * Reads data out of this object into an print stream.
   * 
   * @param dest An {@link PrintStream} to copy data into
   * @param length The maximum number of bytes to attempt to copy
   * @return The number of bytes actually copied, which may be {@code length} or less, including 0,
   *         if less data is available
   */
  public abstract int getBytes(PrintStream dest, int length);

}
