/*
 * Copyright (c) 2013 Peter S. May
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
 * associated documentation files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
 * NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package us.wxy.iomux;

import java.io.IOException;
import java.io.InputStream;


/**
 * Extension of {@link ByteArrayOutput} to include transfers from input streams and byte inputs.
 */
public interface ByteOutput extends ByteArrayOutput {

  /**
   * Writes data into this object from an {@link ByteArrayInput}.
   * 
   * @param src An {@link ByteArrayInput} to copy data from
   * @return The number of bytes actually copied
   */
  public abstract int putBytes(ByteArrayInput src);

  /**
   * Writes data into this object from an {@link ByteArrayInput}.
   * 
   * @param src An {@link ByteArrayInput} to copy data from
   * @param length The maximum number of bytes to attempt to copy
   * @return The number of bytes actually copied
   */
  public abstract int putBytes(ByteArrayInput src, int length);

  /**
   * Writes data into this object from an input stream.
   * 
   * @param src An {@link InputStream} to copy data from
   * @return 0, if length is 0 or there is no remaining write space; otherwise, the return value of
   *         {@code src.read(byte[],int,int)}, which is either the same as the number of bytes
   *         copied or an EOF (implying no bytes were copied).
   * @throws IOException If an I/O error occurs while reading from {@code src}
   */
  public abstract int putBytes(InputStream src) throws IOException;

  /**
   * Writes data into this object from an input stream.
   * 
   * @param src An {@link InputStream} to copy data from
   * @param length The maximum number of bytes to attempt to copy
   * @return 0, if length is 0 or there is no remaining write space; otherwise, the return value of
   *         {@code src.read(byte[],int,int)}, which is either the same as the number of bytes
   *         copied or an EOF (implying no bytes were copied).
   * @throws IOException If an I/O error occurs while reading from {@code src}
   */
  public abstract int putBytes(InputStream src, int length) throws IOException;

}
